function h = DispImagesTiledFromCells(imarray,varargin)
% function DispImagesTiledFromCells(imarray,varargin)
% displays images in same window
%   inputs:
%     imarray: a cell array containing images to be displayed as 2D arrays
% optional inputs: default values and variable names
%    labels: (optional = []) is is a set of labels for the images, uses colors(1) and fontsize
%    fontsize: (optional = 12) The fontsize for labels
%    plot_symbols (optional = '.') symbols to use for points in complex array plots
%    index (optional= 1:nsubjects) is the order to display the images
%    nper_row (optional= 3) number of rows in the display
%    noclip = false; Note: if this is included, won't do the +- 2.5 std dev clip on display
%    displaydat = []; a 2D cell array of same number of rows as imarray of data to plot on the image
%         all columns of the cell array will be plotted
%         Contents can be either an ATLine or a complex array of nx2use-normalized coords
%      Can also be a 1D cell-array of cells. In this case plots all data in 1D cell array
%    colors (optional= 'y')is color of each successive row in displaydat
%    is_nx_normalized: (optional = true) If true, multiplies plot data by image nx2use
%    fill_screen: (optional = true) if true the figures fill the root screen, else matlab default figure size
%  outputs:
%    h: the figure handle
% REA 8/18/06-4/19/08
% PK  Changed nsubjects calc to handle multi-dimensional cell arrays

nreqargs = 1;
assert(nargin>=nreqargs);
assert(iscell(imarray));
nsubjects = numel(imarray);
nper_row = 3;
colors = 'y';
index = 1:nsubjects;
noclip = false;
labels = [];
displaydat = [];
plot_symbols = '.';
is_nx_normalized = true;
fill_screen = true;
fontsize = 12;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'noclip';          noclip = true;
     case 'fill_screen';          fill_screen = varargin{i+1};     i=i+1;
     case 'nper_row';         nper_row=varargin{i+1};     i=i+1;
     case 'index';         index=varargin{i+1};     i=i+1;
     case 'colors';         colors=varargin{i+1};     i=i+1;
     case 'fontsize';         fontsize=varargin{i+1};     i=i+1;
     case 'plot_symbols';         plot_symbols=varargin{i+1};     i=i+1;
     case 'labels';         labels=varargin{i+1}; assert(length(labels)==nsubjects);    i=i+1;
     case 'displaydat';         displaydat=varargin{i+1}; assert(size(displaydat,1)==nsubjects);    i=i+1;
     case 'is_nx_normalized';         is_nx_normalized=varargin{i+1}; assert(length(displaydat)==nsubjects);    i=i+1;
    otherwise
        error('Unknown argument %s ',lower(varargin{i}));
     end
     i=i+1;
  end
end

   if ~isempty(labels)
     if isnumeric(labels)
       labelsn = labels;
       clear labels
       for i = 1:nsubjects
         labels{i} = {num2str(labelsn(i))};
       end %for i
     end %     if isnumeric(labels)
   end % if ~isempty(labels)

if(nsubjects<nper_row)
    nrows = 1;
    ncols = nsubjects;
else    
   nrows = floor(nsubjects/nper_row - 0.01)+1;
   ncols = nper_row;
end
ksubj = 1;
dx = 1/ncols;
dy = 1/nrows;
ycorner = 0;
h = figure;

if fill_screen
    % adjust figure for each computer's display size
  rootSize = get(0,'ScreenSize');
  
  figpos = rootSize(1:2) + [50 50];
  figdim = rootSize(3:4) - [100 150];
  
  set(h,'Position',[figpos figdim]);
end

ksubj = 1;
for krow = 1:nrows
  xcorner = 0;
  for kcol = 1:ncols
    kimage = index(ksubj);
    subplot('Position',[xcorner ycorner dx dy]);
    img = imarray{ksubj};
    [ny,nx2use] = size(img);
    if ~is_nx_normalized
       nx2use = 1;
    end
    if noclip
        imshow(img,[]);
    else
       imsgray(img,false);
    end
    if  ~isempty(labels)
        textoffset = 0.1*[nx2use ny];
        text(textoffset(1),textoffset(2),labels{kimage},'color',colors(1),'fontsize',fontsize);
    end
    if  ~isempty(displaydat)
      if (size(displaydat,2) == 1) && iscell(displaydat(ksubj))
        ddat = displaydat{ksubj};
        for kdat = 1:length(ddat)
          zdat = ddat{kdat};
          if ~isempty(zdat)
             plotsymbol = [plot_symbols(mod(kdat-1,length(plot_symbols))+1) ...
               colors(mod(kdat-1,length(colors))+1)];
             plotdat(zdat,nx2use,plotsymbol)
          end %if ~isempty(zdat)
        end %for kdat = 1:length(ddat)
      else
        for kc = 1:size(displaydat,2)
          zdat = displaydat{ksubj,kc};
          if ~isempty(zdat)
             plotsymbol = [plot_symbols(mod(kc-1,length(plot_symbols))+1) ...
                       colors(mod(kc-1,length(colors))+1)];
             plotdat(zdat,nx2use,plotsymbol)
          end %if ~isempty(zdat)
        end %for kc = 1:size(displaydat,1)
      end % elseif (size(displaydat,2) == 1) && iscell(displaydat(ksubj)
    end %  if  ~isempty(displaydat)
    ksubj = ksubj+1;
    xcorner = xcorner + dx;
    if(ksubj>nsubjects)
        return;
    end
  end %for kcol
  ycorner = ycorner+dy;
end %for krow

function plotdat(zdat,nx2use,plotsymbol)
  if isa(zdat,'ATLine')
    hold on,plot(zdat);hold off
  elseif isa(zdat,'numeric') && ~isreal( zdat )
    hold on,plot(nx2use*zdat,plotsymbol);hold off
  else
    error('data type of displaydat not supported')
  end
