function stats = ImStatsInPolygon(img,zs,varargin)
% function stats = ImStatsInPolygon(img,zs,varargin)
%  Returns statistics of the image data inside a set of polygon
%  inputs:
%    img: an image
%    zs: a cell array of the vertexes of the polygon (x+iy) in image coords
%       can also be a single complex 1D array. If 1D array and only 2 points,
%       assume they are the corners of a rectangle 
%    do_snr: (optional = false), if present, estimates SNR and uses bootstrap to estimate std dev of SNR
%  outputs:
%    stats: an array of struct (1 for each member of the cell array) with fields
%      ave: the average
%      std: the standard deviation
%      skew: the skewness (3d moment) of the data inside the polygon (skew = 0 for a symmetrical distribution)
%      kurtosis: the kurtosis (4th moment) of the data (kurtosis = 3 for Normal distribution)
%      entropy: the entropy of the data inside the polygon
%      imdat: vector of image data inside polygon
%      idxs: indexes to points in the polygon
%  Sample code
% zcorners4mat = 100*[0.3035 + 0.3049i ,  1.0258 + 0.3049i ,  1.0258 + 0.9093i,  0.3035 + 0.9093i; ...
%                           1.5712 + 0.2901i,   2.3009 + 0.2901i,   2.3009 + 1.0714i, 1.5712 + 1.0714i];
% zcorners4 = mat2cell(zcorners4mat,[1 1]); 
% stats = ImStatsInPolygon(img,zcorners4);
% snr = diff([stats.ave])/mean(stats.std);
%  REA 10/7/05-4/1/08 See also ImAveInPolygon

nreqargs = 2;
assert(nargin>=nreqargs);

do_snr = false;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'do_snr'; do_snr = true;
     case 'dets2do';   dets2do=varargin{i+1};   assert(isstruct(dets2do));  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if ~iscell(zs) % its a 1D array
  if numel(zs)==2 % assume opposite corners of rectangle
         % so replace zs with contour around rectangle
      [mnx,mxx] = range2(real(zs));
      [mny,mxy] = range2(imag(zs));
      zs = [mnx+1i*mny, mxx+1i*mny, mxx+1i*mxy, mnx+1i*mxy];
  end
  zs = {zs}; % convert to cell array
end

npolys = length(zs);
stats = struct('ave',ones(npolys,1),'std',ones(npolys,1),'entropy',ones(npolys,1),'kurtosis',ones(npolys,1));
[ny,nx] = size(img);
for kpoly = 1:npolys
  z = zs{kpoly}(:);
  bw = poly2mask(real(z), imag(z), ny, nx );
  idx = find(bw(:)>0);
  imdat = img(idx);
  imdat = imdat(:);
  stats(kpoly).ave = mean(imdat);
  stats(kpoly).std = std(imdat);
  stats(kpoly).skew = skewness(imdat);
  stats(kpoly).kurtosis = kurtosis(imdat);
  [qd,bins] = QuantizeOpt(imdat);
  stats(kpoly).entropy = entropyDIPUM(qd,length(bins{1}));
  stats.imdat = imdat;
  stats(kpoly).idxs = idx;
  if do_snr
         % use bootstrap to compute standard error of SNR
      if abs(stats.std)>eps
          snrs = bootstrp(100, @(x) [mean(x)/std(x)], imdat);
          stats.snrs = snrs;
          stats.snr = mean(snrs);
          stats.snr_se = std(snrs);
      else
          stats.snr = Inf;
          stats.se_snr = 0;
          stats.snrs = [];
      end
  end
end
